<?php

namespace FiloBlu\Refilo\Remote;
use FiloBlu\Refilo\Helper\Indexer;
use FiloBlu\Refilo\Remote\Entity\EntityProviderInterfaceFactory;
use FiloBlu\Refilo\Remote\Helper\TemplateStringFactory;
use FiloBlu\Refilo\Remote\IndexerConfiguration\IndexerConfigurationInterfaceFactory;
use FiloBlu\Refilo\Remote\IndexerConfiguration\IndexerConfigurationRepositoryInterface;
use FiloBlu\Refilo\Remote\IndexerConfiguration\IndexerResolver;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\DataObjectFactory;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Store\Model\StoreManagerInterface;
use Psr\Log\LoggerInterface;

/**
 * Class CategoryIndexer
 * @package FiloBlu\Refilo\Remote
 */
class CategoryIndexer extends AbstractStoreBasedIndexer
{
    /**
     * @var Indexer
     */
    private $indexerHelper;

    /**
     * CategoryIndexer constructor.
     * @param StoreManagerInterface $storeManager
     * @param EntityProviderInterfaceFactory $entityProviderFactory
     * @param IndexerConfigurationRepositoryInterface $indexerConfigurationRepository
     * @param TemplateStringFactory $templateStringFactory
     * @param IndexerResolver $indexerResolver
     * @param ScopeConfigInterface $scopeConfig
     * @param LoggerInterface $logger
     * @param DataObjectFactory $dataObjectFactory
     * @param IndexerConfigurationInterfaceFactory $indexerConfigurationFactory
     * @param Indexer $indexerHelper
     */
    public function __construct(StoreManagerInterface $storeManager,
                                EntityProviderInterfaceFactory $entityProviderFactory,
                                IndexerConfigurationRepositoryInterface $indexerConfigurationRepository,
                                TemplateStringFactory $templateStringFactory,
                                IndexerResolver $indexerResolver,
                                ScopeConfigInterface $scopeConfig,
                                LoggerInterface $logger,
                                DataObjectFactory $dataObjectFactory,
                                IndexerConfigurationInterfaceFactory $indexerConfigurationFactory,
                                Indexer $indexerHelper)
    {
        parent::__construct($storeManager,
            $entityProviderFactory,
            $indexerConfigurationRepository,
            $templateStringFactory,
            $indexerResolver,
            $scopeConfig,
            $logger,
            $dataObjectFactory,
            $indexerConfigurationFactory);
        $this->indexerHelper = $indexerHelper;
    }

    /**
     * @param array $ids
     * @return array
     */
    public function getEntitiesToRemove(array $ids = [])
    {
        return $this->indexerHelper->getDeletedEntityIds('catalog_category_entity', 'entity_id');
    }

    /**
     * @return string
     */
    protected function getClass(): string
    {
        return __CLASS__;
    }

    /**
     * @param $entities
     * @param $collection
     * @throws NoSuchEntityException
     */
    protected function write($entities, $collection)
    {
        $this->getConnector()->create($entities, $collection);
    }
}
